@@ -1,6 +1,6 @@
 /* 
  * 
- * Copyright (C) 2009-2010 Colomban Wendling <ban@herbesfolles.org>
+ * Copyright (C) 2009-2011 Colomban Wendling <ban@herbesfolles.org>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -389,16 +389,13 @@ ensure_cache_filled (CtplInputStream *stream,
   if (stream->buf_pos >= stream->buf_size) {
     gssize read_size;
     
-    /*g_debug ("buffering...");*/
     read_size = g_input_stream_read (stream->stream, stream->buffer,
                                      stream->buf_size, NULL, error);
     if (read_size < 0) {
-      /*g_debug ("failed: %s", error ? (*error)->message : "???");*/
       success = FALSE;
     } else {
       stream->buf_size = (gsize)read_size;
       stream->buf_pos = 0U;
-      /*g_debug ("OK, buffer content is: '%.*s'", stream->buf_size, stream->buffer);*/
     }
   }
   
@@ -429,8 +426,6 @@ resize_cache (CtplInputStream *stream,
   
   g_return_val_if_fail (new_size > 0, FALSE);
   
-  /*g_debug ("resizing queried (from %"G_GSIZE_FORMAT" to %"G_GSIZE_FORMAT" bytes)",
-           stream->buf_size, new_size);*/
   if (new_size > stream->buf_size) {
     gssize read_size;
     gchar *new_buffer;
@@ -564,7 +559,6 @@ ctpl_input_stream_read (CtplInputStream *stream,
         default:
           stream->pos ++;
       }
-      /*g_debug ("read character %d (%c)", c, c);*/
       ((gchar *)buffer)[read_size] = c;
       count --;
     }
@@ -1077,76 +1071,78 @@ ctpl_input_stream_read_number_internal (CtplInputStream *stream,
                                         CtplValue       *value,
                                         GError         **error)
 {
-  gboolean  have_mantissa = FALSE;
-  gboolean  have_exponent = FALSE;
-  gboolean  have_sign     = FALSE;
-  gboolean  have_dot      = FALSE;
-  gint      read_type     = READ_BOTH;
+  gboolean  have_mantissa       = FALSE;
+  gboolean  have_exponent       = FALSE;
+  gboolean  have_exponent_delim = FALSE;
+  gboolean  have_sign           = FALSE;
+  gboolean  have_dot            = FALSE;
   GString  *gstring;
-  GError   *err = NULL;
-  gboolean  in_number = TRUE;
-  gint      base = 10;
+  GError   *err                 = NULL;
+  gboolean  in_number           = TRUE;
+  gint      base                = 10;
+  
+  #define ISSIGN(c)   ((c) == '+' || (c) == '-')
+  #define ISDIGIT(c)  ((c) >= '0' && (c) <= '9')
+  #define ISBDIGIT(c) ((c) == '0' || (c) == '1')
+  #define ISODIGIT(c) ((c) >= '0' && (c) <= '7')
+  #define ISXDIGIT(c) (ISDIGIT (c) || \
+                       ((c) >= 'a' && (c) <= 'f') || \
+                       ((c) >= 'A' && (c) <= 'F'))
   
   gstring = g_string_new ("");
   while (in_number && ! err) {
-    gchar c;
+    gchar   buf[3];
+    gssize  buf_len;
     
-    c = ctpl_input_stream_peek_c (stream, &err);
+    buf_len = ctpl_input_stream_peek (stream, buf, sizeof (buf), &err);
     if (! err) {
-      /*g_debug ("c = %c", c);*/
+      gchar c = (buf_len > 0) ? buf[0] : CTPL_EOF;
+      
       switch (c) {
         case '.':
-          if (have_dot || have_exponent || ! (type & READ_FLOAT)) {
-            in_number = FALSE;
-          } else {
+          if (! have_dot && ! have_exponent_delim && (type & READ_FLOAT)) {
             g_string_append_c (gstring, c);
             have_dot = TRUE;
-            read_type = READ_FLOAT;
+            type &= READ_FLOAT;
+          } else {
+            in_number = FALSE;
           }
           break;
         
         case '+':
         case '-':
-          if ((have_mantissa || have_sign) &&
-              ! (type & READ_FLOAT && have_exponent && gstring->len > 0 &&
-                 (gstring->str[gstring->len - 1] == 'e' ||
-                  gstring->str[gstring->len - 1] == 'p'))) {
-            in_number = FALSE;
-          } else {
+          if (! have_sign && (! have_mantissa ||
+                              (have_exponent_delim && ! have_exponent)) &&
+              /* ISDIGIT() is fine here even though we probably don't know the
+               * base yet because the default base is 10 and the exponent or
+               * power are also in base 10 */
+              buf_len > 1 && ISDIGIT (buf[1])) {
             g_string_append_c (gstring, c);
             have_sign = TRUE;
+          } else {
+            in_number = FALSE;
           }
           break;
         
         case 'e':
         case 'E':
           if (base < 15) {
-            if (have_exponent || ! have_mantissa ||
-                ! (type & READ_FLOAT) || base != 10) {
-              in_number = FALSE;
-            } else {
-              have_exponent = TRUE;
-              read_type = READ_FLOAT;
+            if (have_mantissa && ! have_exponent_delim &&
+                (type & READ_FLOAT) && base == 10 &&
+                ((buf_len > 1 && ISDIGIT (buf[1])) ||
+                 (buf_len > 2 && ISSIGN (buf[1]) && ISDIGIT (buf[2])))) {
+              have_exponent_delim = TRUE;
+              have_sign = FALSE;
+              type &= READ_FLOAT;
               g_string_append_c (gstring, 'e');
+            } else {
+              in_number = FALSE;
             }
             break;
           }
           /* Fallthrough */
         case 'b':
         case 'B':
-          if (type & READ_INT &&
-              ((gstring->len == 1 ||
-                (gstring->len == 2 && (gstring->str[0] == '+' ||
-                                       gstring->str[0] == '-'))) &&
-               gstring->str[gstring->len - 1] == '0')) {
-            /*g_string_append_c (gstring, c);*/
-            have_mantissa = FALSE; /* the previous 0 wasn't mantissa finally */
-            base = 2;
-            read_type = READ_INT;
-            type &= READ_INT;
-            break;
-          }
-          /* Fallthrough */
         case 'a':
         case 'A':
         case 'c':
@@ -1155,7 +1151,7 @@ ctpl_input_stream_read_number_internal (CtplInputStream *stream,
         case 'D':
         case 'f':
         case 'F':
-          if (base < 16 || have_exponent /* exponent is decimal */) {
+          if (base < 16 || have_exponent_delim /* exponent is decimal */) {
             in_number = FALSE;
             break;
           }
@@ -1176,52 +1172,54 @@ ctpl_input_stream_read_number_internal (CtplInputStream *stream,
             in_number = FALSE;
             break;
           }
-        case '0':
-        case '1':
           /* Fallthrough */
-          g_string_append_c (gstring, c);
-          if (! have_exponent) {
-            have_mantissa = TRUE;
+        case '0':
+          if (! have_mantissa && buf_len > 2) {
+            gboolean is_start = TRUE;
+            
+            if ((type & READ_INT) &&
+                (buf[1] == 'b' || buf[1] == 'B') && ISBDIGIT (buf[2])) {
+              type &= READ_INT;
+              base = 2;
+            } else if ((type & READ_INT) &&
+                       (buf[1] == 'o' || buf[1] == 'O') && ISODIGIT (buf[2])) {
+              type &= READ_INT;
+              base = 8;
+            } else if ((buf[1] == 'x' || buf[1] == 'X') && ISXDIGIT (buf[2])) {
+              /* needed for floating-points */
+              g_string_append_c (gstring, c);
+              g_string_append_c (gstring, buf[1]);
+              base = 16;
+            } else {
+              is_start = FALSE;
+            }
+            if (is_start) {
+              /* eat the character we just handled. no need to check the error
+               * since the data is already cached -- so no error can happen */
+              ctpl_input_stream_get_c (stream, NULL);
+              break;
+            }
           }
-          break;
-        
-        case 'o':
-        case 'O':
-          if (type & READ_INT &&
-              ((gstring->len == 1 ||
-                (gstring->len == 2 && (gstring->str[0] == '+' ||
-                                       gstring->str[0] == '-'))) &&
-               gstring->str[gstring->len - 1] == '0')) {
-            /*g_string_append_c (gstring, c);*/
-            have_mantissa = FALSE; /* the previous 0 wasn't mantissa finally */
-            base = 8;
-            read_type = READ_INT;
-            type &= READ_INT;
+          /* Fallthrough */
+        case '1':
+          g_string_append_c (gstring, c);
+          if (! have_exponent_delim) {
+            have_mantissa = TRUE;
           } else {
-            in_number = FALSE;
+            have_exponent = TRUE;
           }
           break;
         
         case 'p':
         case 'P':
-          if (! (type & READ_FLOAT) || have_exponent || base != 16) {
-            in_number = FALSE;
-          } else {
-            have_exponent = TRUE;
-            read_type = READ_FLOAT;
+          if (have_mantissa && ! have_exponent_delim &&
+              (type & READ_FLOAT) && base == 16 &&
+              ((buf_len > 1 && ISDIGIT (buf[1])) ||
+               (buf_len > 2 && ISSIGN (buf[1]) && ISDIGIT (buf[2])))) {
+            have_exponent_delim = TRUE;
+            have_sign = FALSE;
+            type &= READ_FLOAT;
             g_string_append_c (gstring, 'p');
-          }
-          break;
-        
-        case 'x':
-        case 'X':
-          if ((gstring->len == 1 ||
-               (gstring->len == 2 && (gstring->str[0] == '+' ||
-                                      gstring->str[0] == '-'))) &&
-              gstring->str[gstring->len - 1] == '0') {
-            g_string_append_c (gstring, c);
-            have_mantissa = FALSE; /* the previous 0 wasn't mantissa finally */
-            base = 16;
           } else {
             in_number = FALSE;
           }
@@ -1236,48 +1234,40 @@ ctpl_input_stream_read_number_internal (CtplInputStream *stream,
     }
   }
   if (! err) {
-    if (! (read_type & type)) {
+    if (! have_mantissa) {
       ctpl_input_stream_set_error (stream, &err, CTPL_IO_ERROR,
                                    CTPL_IO_ERROR_INVALID_NUMBER,
-                                   "Invalid number");
+                                   "Missing mantissa in numeric constant");
     } else {
-      read_type &= type;
-      if (! have_mantissa) {
+      gchar  *nptr = gstring->str;
+      gchar  *endptr;
+      gdouble dblval = 0.0;
+      glong   longval = 0;
+      gint    errno_save = errno;
+      
+      errno = 0;
+      if (type & READ_INT) {
+        longval = strtol (nptr, &endptr, base);
+      } else {
+        dblval = g_ascii_strtod (nptr, &endptr);
+      }
+      if (! endptr || *endptr != 0) {
         ctpl_input_stream_set_error (stream, &err, CTPL_IO_ERROR,
                                      CTPL_IO_ERROR_INVALID_NUMBER,
-                                     "Missing mantissa in numeric constant");
+                                     "Invalid base %d numeric constant \"%s\"",
+                                     base, nptr);
+      } else if (errno == ERANGE) {
+        ctpl_input_stream_set_error (stream, &err, CTPL_IO_ERROR,
+                                     CTPL_IO_ERROR_RANGE,
+                                     "Overflow in numeric constant conversion");
       } else {
-        gchar  *nptr = gstring->str;
-        gchar  *endptr;
-        gdouble dblval = 0.0;
-        glong   longval = 0;
-        gint    errno_save = errno;
-        
-        /*g_debug ("trying to convert '%s'", nptr);*/
-        errno = 0;
-        if (read_type & READ_INT) {
-          longval = strtol (nptr, &endptr, base);
+        if (type & READ_INT) {
+          ctpl_value_set_int (value, longval);
         } else {
-          dblval = g_ascii_strtod (nptr, &endptr);
+          ctpl_value_set_float (value, dblval);
         }
-        if (! endptr || *endptr != 0) {
-          ctpl_input_stream_set_error (stream, &err, CTPL_IO_ERROR,
-                                       CTPL_IO_ERROR_INVALID_NUMBER,
-                                       "Invalid base %d numeric constant \"%s\"",
-                                       base, nptr);
-        } else if (errno == ERANGE) {
-          ctpl_input_stream_set_error (stream, &err, CTPL_IO_ERROR,
-                                       CTPL_IO_ERROR_RANGE,
-                                       "Overflow in numeric constant conversion");
-        } else {
-          if (read_type & READ_INT) {
-            ctpl_value_set_int (value, longval);
-          } else {
-            ctpl_value_set_float (value, dblval);
-          }
-        }
-        errno = errno_save;
       }
+      errno = errno_save;
     }
   }
   if (err) {
@@ -1285,6 +1275,12 @@ ctpl_input_stream_read_number_internal (CtplInputStream *stream,
   }
   g_string_free (gstring, TRUE);
   
+  #undef ISSIGN
+  #undef ISDIGIT
+  #undef ISBDIGIT
+  #undef ISODIGIT
+  #undef ISXDIGIT
+  
   return ! err;
 }
 
@@ -1382,7 +1378,6 @@ ctpl_input_stream_read_float (CtplInputStream *stream,
     
     c = ctpl_input_stream_peek_c (stream, &err);
     if (! err) {
-      /*g_debug ("c = %c", c);*/
       switch (c) {
         case '.':
           if (have_dot || have_exponent) {
@@ -1489,7 +1484,6 @@ ctpl_input_stream_read_float (CtplInputStream *stream,
       gchar *nptr = gstring->str;
       gchar *endptr;
       
-      /*g_debug ("trying to convert fp '%s'", nptr);*/
       value = g_ascii_strtod (nptr, &endptr);
       if (! endptr || *endptr != 0) {
         ctpl_input_stream_set_error (stream, &err, CTPL_IO_ERROR,
