@@ -1,6 +1,6 @@
 /* 
  * 
- * Copyright (C) 2009-2010 Colomban Wendling <ban@herbesfolles.org>
+ * Copyright (C) 2009-2011 Colomban Wendling <ban@herbesfolles.org>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -93,8 +93,10 @@
  *     <listitem>
  *       <para>
  *         Any numeric constant that ctpl_input_stream_read_number() supports,
- *         or any reference to any 
- *         <link linkend="ctpl-CtplEnviron">environment</link> variable.
+ *         any reference to any
+ *         <link linkend="ctpl-CtplEnviron">environment</link> variable,
+ *         or any string literal that ctpl_input_stream_read_string_literal()
+ *         supports.
  *         An operand may be suffixed with an index of the form
  *         <code>[&lt;expression&gt;]</code>.
  *       </para>
@@ -159,7 +161,6 @@ ctpl_lexer_expr_error_quark (void)
 
 
 /* @stream: #CtplInputStream holding the number.
- * @state: Lexer state
  * @error: return location for errors, or %NULL to ignore them
  * See ctpl_input_stream_read_number()
  * Returns: A new #CtplTokenExpr or %NULL if the input doesn't contain any valid
@@ -167,13 +168,11 @@ ctpl_lexer_expr_error_quark (void)
  */
 static CtplTokenExpr *
 read_number (CtplInputStream *stream,
-             LexerExprState  *state,
              GError         **error)
 {
   CtplTokenExpr  *token = NULL;
   CtplValue       value;
   
-  (void)state; /* we don't use the state, silent compilers */
   ctpl_value_init (&value);
   if (ctpl_input_stream_read_number (stream, &value, error)) {
     token = ctpl_token_expr_new_value (&value);
@@ -188,13 +187,11 @@ read_number (CtplInputStream *stream,
  * Returns: A new #CtplTokenExpr holding the symbol, or %NULL on error */
 static CtplTokenExpr *
 read_symbol (CtplInputStream *stream,
-             LexerExprState  *state,
              GError         **error)
 {
   CtplTokenExpr *token = NULL;
   gchar         *symbol;
   
-  (void)state; /* we don't use the state, silent compilers */
   symbol = ctpl_input_stream_read_symbol (stream, error);
   if (symbol) {
     if (*symbol) {
@@ -210,55 +207,62 @@ read_symbol (CtplInputStream *stream,
   return token;
 }
 
-/* Gets whether op1 has priority over op2.
- * If both operators have the same priority, returns %TRUE */
-static gboolean
-operator_is_prior (CtplOperator op1,
-                   CtplOperator op2)
+/* Reads a string literal from @stream
+ * See ctpl_input_stream_read_string_literal()
+ * Returns: A new #CtplTokenExpr holding the string, or %NULL on error */
+static CtplTokenExpr *
+read_string_literal (CtplInputStream *stream,
+                     GError         **error)
 {
-  if ((op1 == CTPL_OPERATOR_EQUAL || op1 == CTPL_OPERATOR_INF ||
-       op1 == CTPL_OPERATOR_INFEQ || op1 == CTPL_OPERATOR_SUP ||
-       op1 == CTPL_OPERATOR_SUPEQ || op1 == CTPL_OPERATOR_NEQ ||
-       op1 == CTPL_OPERATOR_AND   || op1 == CTPL_OPERATOR_OR) ||
-      ((op1 == CTPL_OPERATOR_MUL || op1 == CTPL_OPERATOR_DIV ||
-        op1 == CTPL_OPERATOR_MODULO) &&
-       ! operator_is_prior (op2, CTPL_OPERATOR_EQUAL)) ||
-      ((op1 == CTPL_OPERATOR_PLUS || op1 == CTPL_OPERATOR_MINUS) &&
-       ! operator_is_prior (op2, CTPL_OPERATOR_MUL))) {
-    return TRUE;
+  CtplTokenExpr *token = NULL;
+  gchar         *string;
+  
+  string = ctpl_input_stream_read_string_literal (stream, error);
+  if (string) {
+    CtplValue value;
+    
+    ctpl_value_init (&value);
+    ctpl_value_set_string (&value, string);
+    token = ctpl_token_expr_new_value (&value);
+    ctpl_value_free_value (&value);
   }
-  return FALSE;
+  g_free (string);
+  
+  return token;
 }
 
 /* 
  * operators_array:
  * 
- * List of operators, with their representation in the CTPL language.
+ * List of operators, with their priority and representation in the CTPL
+ * language.
  * Don't forget to update this when adding an operator */
-/* TODO: make this indexable by the .op field for speed. E.g. this would work:
- * operators_array[i].op == i
- * Hum, not that simple since the order depends on the operator string
- * representation too: >= must come before > to be matched correctly */
+/* This is ordered alphabetically but tweaked for the string representations to be
+ * match-able in order. E.g. >= must come before > not for > to match >=.
+ * 
+ * The order must also be the same as the CtplOperator enum, so
+ * operators_array[i].op == i */
 static const struct {
   CtplOperator  op;       /* The operator ID */
+  guint         priority; /* Op's priority: the higher the prior */
   const gchar  *str;      /* Its string representation */
   gsize         str_len;  /* Cached length of @str */
 } operators_array[] = {
-  { CTPL_OPERATOR_AND,    "&&", 2 },
-  { CTPL_OPERATOR_DIV,    "/",  1 },
-  { CTPL_OPERATOR_EQUAL,  "==", 2 },
-  { CTPL_OPERATOR_INFEQ,  "<=", 2 },
-  { CTPL_OPERATOR_INF,    "<",  1 },
-  { CTPL_OPERATOR_MINUS,  "-",  1 },
-  { CTPL_OPERATOR_MODULO, "%",  1 },
-  { CTPL_OPERATOR_MUL,    "*",  1 },
-  { CTPL_OPERATOR_NEQ,    "!=", 2 },
-  { CTPL_OPERATOR_OR,     "||", 2 },
-  { CTPL_OPERATOR_PLUS,   "+",  1 },
-  { CTPL_OPERATOR_SUPEQ,  ">=", 2 },
-  { CTPL_OPERATOR_SUP,    ">",  1 },
+  { CTPL_OPERATOR_AND,    30, "&&", 2 },
+  { CTPL_OPERATOR_DIV,    20, "/",  1 },
+  { CTPL_OPERATOR_EQUAL,  30, "==", 2 },
+  { CTPL_OPERATOR_INFEQ,  30, "<=", 2 },
+  { CTPL_OPERATOR_INF,    30, "<",  1 },
+  { CTPL_OPERATOR_MINUS,  10, "-",  1 },
+  { CTPL_OPERATOR_MODULO, 20, "%",  1 },
+  { CTPL_OPERATOR_MUL,    20, "*",  1 },
+  { CTPL_OPERATOR_NEQ,    30, "!=", 2 },
+  { CTPL_OPERATOR_OR,     30, "||", 2 },
+  { CTPL_OPERATOR_PLUS,   10, "+",  1 },
+  { CTPL_OPERATOR_SUPEQ,  30, ">=", 2 },
+  { CTPL_OPERATOR_SUP,    30, ">",  1 },
   /* must be last */
-  { CTPL_OPERATOR_NONE,   "not an operator", 15 }
+  { CTPL_OPERATOR_NONE,   0,  "not an operator", 15 }
 };
 /* number of true operators, without the NONE at the end */
 static const gsize operators_array_length = G_N_ELEMENTS (operators_array) - 1;
@@ -266,6 +270,15 @@ static const gsize operators_array_length = G_N_ELEMENTS (operators_array) - 1;
 #define            OPERATORS_STR_MAXLEN     (2)
 
 
+/* Gets whether op1 has priority over op2.
+ * If both operators have the same priority, returns %TRUE */
+static gboolean
+operator_is_prior (CtplOperator op1,
+                   CtplOperator op2)
+{
+  return operators_array[op1].priority >= operators_array[op2].priority;
+}
+
 /*
  * ctpl_operator_to_string:
  * @op: A #CtplOperator
@@ -279,17 +292,9 @@ static const gsize operators_array_length = G_N_ELEMENTS (operators_array) - 1;
 const gchar *
 ctpl_operator_to_string (CtplOperator op)
 {
-  gsize i = operators_array_length; /* by default, index the last op (error) */
+  g_return_val_if_fail (op >= 0 && op <= CTPL_OPERATOR_NONE, NULL);
   
-  /* if not an operator, final incrementation leads to index (n_ops + 1),
-   * the error message; otherwise, we break then i indexes the operator. */
-  for (i = 0; i < operators_array_length; i++) {
-    if (operators_array[i].op == op) {
-      break;
-    }
-  }
-  
-  return operators_array[i].str;
+  return operators_array[op].str;
 }
 
 /*
@@ -369,29 +374,22 @@ validate_token_list (CtplInputStream *stream,
   gint            opt = 0;
   gint            opd = 0;
   gint            i = 0;
-  GSList         *tmp;
   GSList         *last_opd = NULL;
   
-  /*g_debug ("validating tokens...");*/
   /* we can assume the token list is fully valid, never having invalid token
    * suits as the caller have checked it, except for the last token that may be
    * an operator (then which misses its right operand) */
-  for (tmp = tokens; tokens; tokens = tokens->next) {
+  for (; tokens; tokens = tokens->next) {
     if ((i++ % 2) == 0) {
       operands[opd++] = tokens->data;
       last_opd = tokens;
-      /*g_debug ("found an operand:");
-      ctpl_token_expr_dump (operands[opd-1]);*/
     } else {
       operators[opt++] = tokens->data;
-      /*g_debug ("found an operator:");
-      ctpl_token_expr_dump (operands[opt-1]);*/
       if (opt > 1) {
         if (operator_is_prior (operators[0]->token.t_operator->operator,
                                operators[1]->token.t_operator->operator)) {
           CtplTokenExpr *nexpr;
           
-          /*g_debug ("Operator is prior");*/
           nexpr = operators[0];
           operators[0] = operators[1];
           operators[1] = NULL;
@@ -402,7 +400,6 @@ validate_token_list (CtplInputStream *stream,
           operands[1] = NULL;
           opd = 1;
         } else {
-          /*g_debug ("Operator is not prior");*/
           operands[1] = validate_token_list (stream, last_opd, error);
           opt --;
           
@@ -429,7 +426,6 @@ validate_token_list (CtplInputStream *stream,
                                  "Too few operands for operator '%s'",
                                  token_operator_to_string (operators[opt - 1]));
   }
-  /*_debug ("done.");*/
   
   return expr;
 }
@@ -442,8 +438,8 @@ lex_operand_index (CtplInputStream *stream,
   gboolean success = TRUE;
   
   /* if we have something that looks like an index, try to read it */
-  /* FIXME: handle blanks before the index */
-  while (success && ctpl_input_stream_peek_c (stream, NULL) == '[') {
+  while (success && ctpl_input_stream_skip_blank (stream, error) >= 0 &&
+         ctpl_input_stream_peek_c (stream, NULL) == '[') {
     CtplTokenExpr  *idx;
     
     success = FALSE;
@@ -478,7 +474,6 @@ lex_operand_index (CtplInputStream *stream,
  * Returns: A new #CtplTokenExpr on success, %NULL on error. */
 static CtplTokenExpr *
 lex_operand (CtplInputStream *stream,
-             LexerExprState  *state,
              GError         **error)
 {
   CtplTokenExpr  *token = NULL;
@@ -493,9 +488,11 @@ lex_operand (CtplInputStream *stream,
     if (g_ascii_isdigit (c) ||
         (c == '.' && g_ascii_isdigit (next_c)) ||
         c == '+' || c == '-') {
-      token = read_number (stream, state, error);
+      token = read_number (stream, error);
     } else if (ctpl_is_symbol (c)) {
-      token = read_symbol (stream, state, error);
+      token = read_symbol (stream, error);
+    } else if (c == CTPL_STRING_DELIMITER_CHAR) {
+      token = read_string_literal (stream, error);
     } else {
       ctpl_input_stream_set_error (stream, error, CTPL_LEXER_EXPR_ERROR,
                                    CTPL_LEXER_EXPR_ERROR_SYNTAX_ERROR,
@@ -513,7 +510,6 @@ lex_operand (CtplInputStream *stream,
  * Returns: A new #CtplTokenExpr on success, %NULL on error. */
 static CtplTokenExpr *
 lex_operator (CtplInputStream *stream,
-              LexerExprState  *state,
               GError         **error)
 {
   CtplTokenExpr  *token   = NULL;
@@ -522,10 +518,8 @@ lex_operator (CtplInputStream *stream,
   gchar           buf[OPERATORS_STR_MAXLEN];
   gssize          read_size;
   
-  (void)state; /* we don't use the state, silent compilers */
   read_size = ctpl_input_stream_peek (stream, buf, sizeof buf, error);
   if (read_size >= 0) {
-    /*g_debug ("Lexing operator '%.*s'", sizoef buf, buf);*/
     op = ctpl_operator_from_string (buf, read_size, &off);
     if (op == CTPL_OPERATOR_NONE) {
       ctpl_input_stream_set_error (stream, error, CTPL_LEXER_EXPR_ERROR,
@@ -595,16 +589,16 @@ ctpl_lexer_expr_lex_internal (CtplInputStream  *stream,
                 }
               }
             } else {
-              token = lex_operand (stream, state, &err);
+              token = lex_operand (stream, &err);
             }
           } else {
             /* try to read an operator */
-            token = lex_operator (stream, state, &err);
+            token = lex_operator (stream, &err);
           }
         }
         if (token) {
           expect_operand = ! expect_operand;
-          tokens = g_slist_append (tokens, token);
+          tokens = g_slist_prepend (tokens, token);
         } else if (! state->lex_all) {
           if (err->domain != CTPL_IO_ERROR) {
             /* if we don't validate all, we don't want to throw an error when no
@@ -625,6 +619,7 @@ ctpl_lexer_expr_lex_internal (CtplInputStream  *stream,
                                      "No valid operand at start of expression");
       } else {
         /* here check validity of token list, then create the final token. */
+        tokens = g_slist_reverse (tokens);
         expr_tok = validate_token_list (stream, tokens, &err);
       }
     }
