@@ -1,6 +1,6 @@
 /* 
  * 
- * Copyright (C) 2009-2010 Colomban Wendling <ban@herbesfolles.org>
+ * Copyright (C) 2009-2011 Colomban Wendling <ban@herbesfolles.org>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -420,8 +420,9 @@ ctpl_value_set_array_internal (CtplValue     *value,
   GSList *new_values = NULL;
   
   for (; values != NULL; values = values->next) {
-    new_values = g_slist_append (new_values, ctpl_value_dup (values->data));
+    new_values = g_slist_prepend (new_values, ctpl_value_dup (values->data));
   }
+  new_values = g_slist_reverse (new_values);
   ctpl_value_free_value (value);
   value->type = CTPL_VTYPE_ARRAY;
   value->value.v_array = new_values;
@@ -970,7 +971,6 @@ ctpl_value_get_array_int (const CtplValue *value,
     }
   }
   
- success:
   if (length) *length = len;
   return array;
  fail:
@@ -1015,7 +1015,6 @@ ctpl_value_get_array_float (const CtplValue *value,
     }
   }
   
- success:
   if (length) *length = len;
   return array;
  fail:
@@ -1063,7 +1062,6 @@ ctpl_value_get_array_string (const CtplValue *value,
   }
   array[n] = NULL;
   
- success:
   if (length) *length = len;
   return array;
  fail:
@@ -1120,9 +1118,7 @@ ctpl_value_to_string (const CtplValue *value)
     }
     
     case CTPL_VTYPE_FLOAT:
-      val = g_ascii_dtostr (g_malloc (G_ASCII_DTOSTR_BUF_SIZE),
-                            G_ASCII_DTOSTR_BUF_SIZE,
-                            value->value.v_float);
+      val = ctpl_math_float_to_string (value->value.v_float);
       break;
     
     case CTPL_VTYPE_INT:
