@@ -1,6 +1,6 @@
 /* 
  * 
- * Copyright (C) 2009-2010 Colomban Wendling <ban@herbesfolles.org>
+ * Copyright (C) 2009-2011 Colomban Wendling <ban@herbesfolles.org>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -60,7 +60,7 @@ static GOptionEntry option_entries[] = {
     "Specify the encoding of the input and output files.", "ENCODING" },
   { G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_FILENAME_ARRAY, &OPT_input_files,
     "Input files", "INPUTFILE[...]" },
-  { NULL }
+  { NULL, 0, 0, G_OPTION_ARG_NONE, NULL, NULL, NULL }
 };
 
 
